@@ -15,7 +15,7 @@
 // [[Rcpp::depends(RcppThread)]]
 
 /**
- * @brief Computes the partial correlation between a spatial cross-section series and its prediction
+ * @brief Computes the partial correlation between a spatial cross-sectional series and its prediction
  *        using the Simplex Projection method, incorporating control variables in a grid-based spatial setting.
  *
  * This function reconstructs the state-space and applies Simplex Projection prediction while accounting
@@ -23,10 +23,10 @@
  * control variables.
  *
  * @param vectors: Reconstructed state-space where each row represents a separate vector/state.
- * @param target: Spatial cross-section series used as the prediction target.
+ * @param target: Spatial cross-sectional series used as the prediction target.
  * @param controls: Cross-sectional data of control variables, stored row-wise.
- * @param lib_indices: Boolean vector indicating which states to include when searching for neighbors.
- * @param pred_indices: Boolean vector indicating which states to predict from.
+ * @param lib_indices: Integer vector indicating which states to include when searching for neighbors.
+ * @param pred_indices: Integer vector indicating which states to predict from.
  * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
  * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
  * @param num_neighbors: Vector specifying the numbers of neighbors to use for Simplex Projection.
@@ -40,8 +40,8 @@ std::vector<double> PartialSimplex4Grid(
     const std::vector<std::vector<double>>& vectors,
     const std::vector<double>& target,
     const std::vector<std::vector<double>>& controls,
-    const std::vector<bool>& lib_indices,
-    const std::vector<bool>& pred_indices,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
@@ -98,7 +98,7 @@ std::vector<double> PartialSimplex4Grid(
 }
 
 /**
- * @brief Computes the partial correlation between a spatial cross-section series and its prediction
+ * @brief Computes the partial correlation between a spatial cross-sectional series and its prediction
  *        using the S-Map method, incorporating control variables in a grid-based spatial setting.
  *
  * This function reconstructs the state-space and applies S-Map prediction while accounting for
@@ -106,10 +106,10 @@ std::vector<double> PartialSimplex4Grid(
  * control variables.
  *
  * @param vectors: Reconstructed state-space where each row represents a separate vector/state.
- * @param target: Spatial cross-section series used as the prediction target.
+ * @param target: Spatial cross-sectional series used as the prediction target.
  * @param controls: Cross-sectional data of control variables, stored row-wise.
- * @param lib_indices: Boolean vector indicating which states to include when searching for neighbors.
- * @param pred_indices: Boolean vector indicating which states to predict from.
+ * @param lib_indices: Integer vector indicating which states to include when searching for neighbors.
+ * @param pred_indices: Integer vector indicating which states to predict from.
  * @param conEs: Vector specifying the number of dimensions for attractor reconstruction with control variables.
  * @param taus: Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
  * @param num_neighbors: Vector specifying the numbers of neighbors to use for Simplex Projection.
@@ -124,8 +124,8 @@ std::vector<double> PartialSMap4Grid(
     const std::vector<std::vector<double>>& vectors,
     const std::vector<double>& target,
     const std::vector<std::vector<double>>& controls,
-    const std::vector<bool>& lib_indices,
-    const std::vector<bool>& pred_indices,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& num_neighbors,
@@ -188,12 +188,12 @@ std::vector<double> PartialSMap4Grid(
  * This function calculates the partial cross mapping between a predictor variable (xEmbedings) and a response
  * variable (yPred) over a 2D grid, using either Simplex Projection or S-Mapping.
  *
- * @param xEmbedings           A 2D matrix of the predictor variable's embeddings (spatial cross-section data).
- * @param yPred                A 1D vector of the response variable's values (spatial cross-section data).
+ * @param xEmbedings           A 2D matrix of the predictor variable's embeddings (spatial cross-sectional data).
+ * @param yPred                A 1D vector of the response variable's values (spatial cross-sectional data).
  * @param controls             A 2D matrix that stores the control variables.
  * @param lib_sizes            A vector of two integers, where the first element is the row-wise library size and the second element is the column-wise library size.
  * @param possible_lib_indices A boolean vector indicating which spatial units are valid for inclusion in the library.
- * @param pred_indices         A boolean vector indicating which spatial units to be predicted.
+ * @param pred_indices         A integer vector indicating which spatial units to be predicted.
  * @param conEs                Number of dimensions for the attractor reconstruction with control variables
  * @param taus:                Vector specifying the spatial lag step for constructing lagged state-space vectors with control variables.
  * @param totalRow             The total number of rows in the 2D grid.
@@ -214,7 +214,7 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
     const std::vector<std::vector<double>>& controls,
     const std::vector<int>& lib_sizes,
     const std::vector<bool>& possible_lib_indices,
-    const std::vector<bool>& pred_indices,
+    const std::vector<int>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& b,
@@ -233,7 +233,6 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
 
   // Determine the marked libsize
   const int libsize = row_size_mark ? lib_size_row : lib_size_col;
-  const int half_lib_size = (lib_size_row * lib_size_col) / 2;
 
   // Precompute valid (r, c) pairs
   std::vector<std::pair<int, int>> valid_indices;
@@ -258,17 +257,13 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
     const int c = valid_indices[idx].second;
 
     // Initialize library indices and count the number of the nan value together
-    std::vector<bool> lib_indices(totalRow * totalCol, false);
-    int na_count = 0;
+    std::vector<int> lib_indices;
 
     for (int ii = r; ii < r + lib_size_row; ++ii) {
       for (int jj = c; jj < c + lib_size_col; ++jj) {
-        const int index = (ii - 1) * totalCol + (jj - 1);
+        int index = (ii - 1) * totalCol + (jj - 1);
         if (possible_lib_indices[index]) {
-          lib_indices[index] = true;
-          if (std::isnan(yPred[index])) {
-            ++na_count;
-          }
+          lib_indices.emplace_back(index);
         }
       }
     }
@@ -276,13 +271,11 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
     // Directly initialize std::vector<double> with two NaN values
     std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
 
-    if (na_count <= half_lib_size) {
-      // Run partial cross map and store results
-      if (simplex) {
-        rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
-      } else {
-        rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
-      }
+    // Run partial cross map and store results
+    if (simplex) {
+      rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
+    } else {
+      rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
     }
 
     // Directly assign a PartialCorRes struct with the three values
@@ -308,12 +301,11 @@ std::vector<PartialCorRes> SCPCMSingle4Grid(
  * are created by selecting consecutive indices from possible_lib_indices with possible wraparound.
  *
  * @param xEmbedings           State-space embeddings for the predictor variable (each row is a spatial vector)
- * @param yPred                Target spatial cross-section series
+ * @param yPred                Target spatial cross-sectional series
  * @param controls             Control variables stored by row
  * @param lib_size             Number of consecutive spatial units to include in each library
- * @param max_lib_size         Maximum possible library size (total valid spatial units)
- * @param possible_lib_indices Integer vector indicating the indices of eligible spatial units for library construction
- * @param pred_indices         Boolean vector indicating spatial units to predict
+ * @param lib_indices          Integer vector indicating the indices of eligible spatial units for library construction
+ * @param pred_indices         Integer vector indicating spatial units to predict
  * @param conEs                Embedding dimensions for control variables
  * @param taus                 Spatial lag steps for control variable embeddings
  * @param b                    Number of nearest neighbors for prediction
@@ -332,9 +324,8 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
     const std::vector<double>& yPred,
     const std::vector<std::vector<double>>& controls,
     int lib_size,
-    int max_lib_size,
-    const std::vector<int>& possible_lib_indices,
-    const std::vector<bool>& pred_indices,
+    const std::vector<int>& lib_indices,
+    const std::vector<int>& pred_indices,
     const std::vector<int>& conEs,
     const std::vector<int>& taus,
     const std::vector<int>& b,
@@ -345,32 +336,17 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
     size_t threads,
     int parallel_level,
     bool cumulate) {
-  int n = yPred.size();
+  int max_lib_size = lib_indices.size();
 
   // No possible library variation if using all vectors
   if (lib_size == max_lib_size) {
-    std::vector<bool> lib_indices(n, false);
-    for (int idx : possible_lib_indices) {
-      lib_indices[idx] = true;
-    }
-
-    // Check if more than half of the library is NA
-    int na_count = 0;
-    for (size_t i = 0; i < lib_indices.size(); ++i) {
-      if (lib_indices[i] && std::isnan(yPred[i])) {
-        ++na_count;
-      }
-    }
-
     std::vector<PartialCorRes> x_xmap_y;
     std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
-    if (na_count <= max_lib_size / 2.0) {
-      // Run partial cross map and store results
-      if (simplex) {
-        rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
-      } else {
-        rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
-      }
+    // Run partial cross map and store results
+    if (simplex) {
+      rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
+    } else {
+      rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
     }
 
     x_xmap_y.emplace_back(lib_size, rho[0], rho[1]);
@@ -383,15 +359,15 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
       // Loop around to beginning of lib indices
       if (start_lib + lib_size > max_lib_size) {
         for (int i = start_lib; i < max_lib_size; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
         int num_vectors_remaining = lib_size - (max_lib_size - start_lib);
         for (int i = 0; i < num_vectors_remaining; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
       } else {
         for (int i = start_lib; i < start_lib + lib_size; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
       }
       valid_lib_indices.emplace_back(local_lib_indices);
@@ -402,30 +378,13 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
 
     // Perform the operations using RcppThread
     RcppThread::parallelFor(0, valid_lib_indices.size(), [&](size_t i) {
-      std::vector<bool> lib_indices(n, false);
-      std::vector<int> local_lib_indices = valid_lib_indices[i];
-      for(int& li : local_lib_indices){
-        lib_indices[possible_lib_indices[li]] = true;
-      }
-
-      // Check if more than half of the library is NA
-      int na_count = 0;
-      for (size_t ii = 0; ii < lib_indices.size(); ++ii) {
-        if (lib_indices[ii] && std::isnan(yPred[ii])) {
-          ++na_count;
-        }
-      }
-
       std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
-      if (na_count <= max_lib_size / 2.0) {
-        // Run partial cross map and store results
-        if (simplex) {
-          rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
-        } else {
-          rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
-        }
+      // Run partial cross map and store results
+      if (simplex) {
+        rho = PartialSimplex4Grid(xEmbedings, yPred, controls, valid_lib_indices[i], pred_indices, conEs, taus, b, totalRow, cumulate);
+      } else {
+        rho = PartialSMap4Grid(xEmbedings, yPred, controls, valid_lib_indices[i], pred_indices, conEs, taus, b, totalRow, theta, cumulate);
       }
-
       // Directly initialize a PartialCorRes struct with the three values
       PartialCorRes result(lib_size, rho[0], rho[1]);
       x_xmap_y[i] = result;
@@ -440,15 +399,15 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
       // Loop around to beginning of lib indices
       if (start_lib + lib_size > max_lib_size) {
         for (int i = start_lib; i < max_lib_size; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
         int num_vectors_remaining = lib_size - (max_lib_size - start_lib);
         for (int i = 0; i < num_vectors_remaining; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
       } else {
         for (int i = start_lib; i < start_lib + lib_size; ++i) {
-          local_lib_indices.emplace_back(i);
+          local_lib_indices.emplace_back(lib_indices[i]);
         }
       }
       valid_lib_indices.emplace_back(local_lib_indices);
@@ -459,30 +418,13 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
 
     // Iterate through precomputed valid_lib_indices
     for (size_t i = 0; i < valid_lib_indices.size(); ++i){
-      std::vector<bool> lib_indices(n, false);
-      std::vector<int> local_lib_indices = valid_lib_indices[i];
-      for(int& li : local_lib_indices){
-        lib_indices[possible_lib_indices[li]] = true;
-      }
-
-      // Check if more than half of the library is NA
-      int na_count = 0;
-      for (size_t ii = 0; ii < lib_indices.size(); ++ii) {
-        if (lib_indices[ii] && std::isnan(yPred[ii])) {
-          ++na_count;
-        }
-      }
-
       std::vector<double> rho(2, std::numeric_limits<double>::quiet_NaN());
-      if (na_count <= max_lib_size / 2.0) {
-        // Run partial cross map and store results
-        if (simplex) {
-          rho = PartialSimplex4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, cumulate);
-        } else {
-          rho = PartialSMap4Grid(xEmbedings, yPred, controls, lib_indices, pred_indices, conEs, taus, b, totalRow, theta, cumulate);
-        }
+      // Run partial cross map and store results
+      if (simplex) {
+        rho = PartialSimplex4Grid(xEmbedings, yPred, controls, valid_lib_indices[i], pred_indices, conEs, taus, b, totalRow, cumulate);
+      } else {
+        rho = PartialSMap4Grid(xEmbedings, yPred, controls, valid_lib_indices[i], pred_indices, conEs, taus, b, totalRow, theta, cumulate);
       }
-
       // Directly initialize a PartialCorRes struct with the three values
       PartialCorRes result(lib_size, rho[0], rho[1]);
       x_xmap_y[i] = result;
@@ -500,8 +442,8 @@ std::vector<PartialCorRes> SCPCMSingle4GridOneDim(
  * and S-Mapping, with options for parallel computation and progress tracking.
  *
  * Parameters:
- * - xMatrix: A 2D matrix of predictor variable values (spatial cross-section data).
- * - yMatrix: A 2D matrix of response variable values (spatial cross-section data).
+ * - xMatrix: A 2D matrix of predictor variable values (spatial cross-sectional data).
+ * - yMatrix: A 2D matrix of response variable values (spatial cross-sectional data).
  * - zMatrixs: A 2D matrix storing the control variables.
  * - lib_sizes: A 2D vector where the first sub-vector contains row-wise library sizes and the second sub-vector contains column-wise library sizes.
  * - lib: A vector of pairs representing the indices (row, column) of spatial units to be the library.
@@ -578,7 +520,7 @@ std::vector<std::vector<double>> SCPCM4Grid(
   // Generate embeddings for xMatrix
   std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, Ex, taux);
 
-  int n_confounds;
+  size_t n_confounds;
   if (cumulate){
     n_confounds = 1;
   } else {
@@ -648,18 +590,10 @@ std::vector<std::vector<double>> SCPCM4Grid(
     lib_indices[LocateGridIndices(l.first + 1, l.second + 1, totalRow, totalCol)] = true;
   }
 
-  // Set prediction indices
-  std::vector<bool> pred_indices(totalRow * totalCol, false);
+  // Set predict indices
+  std::vector<int> pred_indices;
   for (const auto& p : pred) {
-    pred_indices[LocateGridIndices(p.first + 1, p.second + 1, totalRow, totalCol)] = true;
-  }
-
-  // Exclude NA values in yPred from the library and prediction indices
-  for (size_t i = 0; i < yPred.size(); ++i) {
-    if (std::isnan(yPred[i])) {
-      lib_indices[i] = false;
-      pred_indices[i] = false;
-    }
+    pred_indices.push_back(LocateGridIndices(p.first + 1, p.second + 1, totalRow, totalCol));
   }
 
   // Local results for each library
@@ -802,7 +736,7 @@ std::vector<std::vector<double>> SCPCM4Grid(
     final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
   }
 
-  int n = pred.size();
+  size_t n = pred.size();
   // Compute significance and confidence intervals for each result
   for (size_t i = 0; i < final_results.size(); ++i) {
     double rho_second = final_results[i][1];
@@ -837,8 +771,8 @@ std::vector<std::vector<double>> SCPCM4Grid(
  * - yMatrix: 2D grid of target variable values (row-major order)
  * - zMatrixs: Control variables stored as 2D grids (vector of vectors)
  * - lib_size: Number of consecutive spatial units to include in each library
- * - lib: Vector specifying library location as (row, col) pairs
- * - pred: Vector specifying prediction locations
+ * - lib: Vector specifying library location as raster(grid) cell indices in row-major order
+ * - pred: Vector specifying prediction location as raster(grid) cell indices in row-major order
  * - Es: Embedding dimensions for x and control variables
  * - taus: Spatial lag steps for x and control variables
  * - b: Numbers of nearest neighbors for prediction
@@ -911,31 +845,14 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
   // Generate embeddings for xMatrix
   std::vector<std::vector<double>> xEmbedings = GenGridEmbeddings(xMatrix, Ex, taux);
 
-  int n_confounds;
+  size_t n_confounds;
   if (cumulate){
     n_confounds = 1;
   } else {
     n_confounds = zMatrixs.size();
   }
 
-  std::vector<int> possible_lib_indices;
-  for (size_t i = 0; i < lib.size(); ++i) {
-    int LibIndice = lib[i] - 1;
-    if (!std::isnan(yPred[LibIndice])) {
-      possible_lib_indices.push_back(LibIndice);
-    }
-  }
-  int max_lib_size = static_cast<int>(possible_lib_indices.size()); // Maximum lib size
-
-  // Initialize pred_indices with all false
-  std::vector<bool> pred_indices(totalRow*totalCol, false);
-  // Convert pred (1-based in R) to 0-based indices, exclude yPred NA and set corresponding positions to true
-  for (size_t i = 0; i < pred.size(); ++i) {
-    int PreIndice = pred[i] - 1;
-    if (!std::isnan(yPred[PreIndice])) {
-      pred_indices[PreIndice] = true;
-    }
-  }
+  int max_lib_size = static_cast<int>(lib.size()); // Maximum lib size
 
   std::vector<int> unique_lib_sizes(lib_sizes.begin(), lib_sizes.end());
 
@@ -964,9 +881,8 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           yPred,
           zMatrixs,
           unique_lib_sizes[i],
-          max_lib_size,
-          possible_lib_indices,
-          pred_indices,
+          lib,
+          pred,
           conEs,
           contaus,
           bs,
@@ -987,9 +903,8 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           yPred,
           zMatrixs,
           unique_lib_sizes[i],
-          max_lib_size,
-          possible_lib_indices,
-          pred_indices,
+          lib,
+          pred,
           conEs,
           contaus,
           bs,
@@ -1014,9 +929,8 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           yPred,
           zMatrixs,
           lib_size,
-          max_lib_size,
-          possible_lib_indices,
-          pred_indices,
+          lib,
+          pred,
           conEs,
           contaus,
           bs,
@@ -1038,9 +952,8 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
           yPred,
           zMatrixs,
           lib_size,
-          max_lib_size,
-          possible_lib_indices,
-          pred_indices,
+          lib,
+          pred,
           conEs,
           contaus,
           bs,
@@ -1088,7 +1001,7 @@ std::vector<std::vector<double>> SCPCM4GridOneDim(
     final_results.push_back({static_cast<double>(group.first), mean_second, mean_third});
   }
 
-  int n = pred.size();
+  size_t n = pred.size();
   // Compute significance and confidence intervals for each result
   for (size_t i = 0; i < final_results.size(); ++i) {
     double rho_second = final_results[i][1];
